<?php
/**
 * Slash Admin Options page
 * 
 * @package Slash Admin
 * @since 1.0
 */

class Slash_Admin_Options {
	
	private $sections;
	private $checkboxes;
	private $settings;
	
	/**
	 * Construct
	 *
	 * @since 1.0
	 */
	public function __construct() {
		
		// This will keep track of the checkbox options for the validate_settings function.
		$this->checkboxes = array();
		$this->settings = array();
		$this->get_settings();
		
		$this->sections['appearance']   		= __( 'Frontend' , 'slashadmin' );
		$this->sections['administration']   	= __( 'Administration' , 'slashadmin' );
		$this->sections['frontend_usability'] 	= __( 'Login screen' , 'slashadmin' );
		$this->sections['backend_usability'] 	= __( 'Non-admins' , 'slashadmin' );
		$this->sections['white_label']   		= __( 'White label' , 'slashadmin' );
		$this->sections['performance']   		= __( 'Performance' , 'slashadmin' );
		$this->sections['shortcodes']   		= __( 'Shortcodes' , 'slashadmin' );
		$this->sections['about']     			= __( 'Documentation' , 'slashadmin' );
		
		add_action( 'admin_menu', array( &$this, 'add_pages' ) );
		add_action( 'admin_init', array( &$this, 'register_settings' ) );
		
		if ( ! get_option( 'slashadmin_options' ) )
			$this->initialize_settings();
		
	}
	
	/**
	 * Add options page
	 *
	 * @since 1.0
	 */
	public function add_pages() {
		
		$admin_page = add_management_page( __( 'Slash Admin Options' , 'slashadmin' ), __( 'Slash Admin' , 'slashadmin' ), 'manage_options', 'slashadmin-options', array( &$this, 'display_page' ) );
		
		add_action( 'admin_print_scripts-' . $admin_page, array( &$this, 'slashadmin_scripts' ) );
		add_action( 'admin_print_styles-' . $admin_page, array( &$this, 'styles' ) );
		
	}
	
	/**
	 * Create settings field
	 *
	 * @since 1.0
	 */
	public function create_setting( $args = array() ) {
		
		$defaults = array(
			'id'      => 'default_field',
			'title'   => __( 'Default Field' , 'slashadmin' ),
			'desc'    => __( 'This is a default description.' , 'slashadmin' ),
			'std'     => '',
			'type'    => 'text',
			'section' => 'general',
			'choices' => array(),
			'class'   => '',
			'para' => ''
		);
			
		extract( wp_parse_args( $args, $defaults ) );
		
		$field_args = array(
			'type'      => $type,
			'id'        => $id,
			'desc'      => $desc,
			'std'       => $std,
			'choices'   => $choices,
			'label_for' => $id,
			'class'     => $class,
			'paragraph' => $para
		);
		
		if ( $type == 'checkbox' )
			$this->checkboxes[] = $id;
		
		add_settings_field( $id, $title, array( $this, 'display_setting' ), 'slashadmin-options', $section, $field_args );
	}
	
	/**
	 * Display options page
	 *
	 * @since 1.0
	 */
	public function display_page() {
		
		echo '<div class="wrap">
	<div class="icon32" id="icon-options-general"></div>
	<h1 class="slash-admin-header">' . __( 'Slash Admin Options' , 'slashadmin' ) . '</h1>';
	
		if ( isset( $_GET['settings-updated'] ) && $_GET['settings-updated'] == true )
			echo '<div class="updated fade"><p>' . __( 'Options updated.' , 'slashadmin' ) . '</p></div>';
		
		echo '<form action="options.php" method="post">';
	
		settings_fields( 'slashadmin_options' );
		echo '<div class="ui-tabs">
			<ul class="ui-tabs-nav">';
		
		foreach ( $this->sections as $section_slug => $section )
			echo '<li><a href="#' . $section_slug . '">' . $section . '</a></li>';
		
		echo '</ul>';

		do_settings_sections( $_GET['page'] );
		
		echo '</div>
		<p class="submit"><input name="Submit" type="submit" class="button-primary" value="' . __( 'Save Changes' , 'slashadmin' ) . '" /></p>
		
	</form>';
	
	echo '<script type="text/javascript">
		jQuery(document).ready(function($) {
			var sections = [];';
			
			foreach ( $this->sections as $section_slug => $section )
				echo "sections['$section'] = '$section_slug';";
			
			echo 'var wrapped = $(".wrap h3").wrap("<div class=\"ui-tabs-panel\">");
			wrapped.each(function() {
				$(this).parent().append($(this).parent().nextUntil("div.ui-tabs-panel"));
			});
			$(".ui-tabs-panel").each(function(index) {
				$(this).attr("id", sections[$(this).children("h3").text()]);
				if (index > 0)
					$(this).addClass("ui-tabs-hide");
			});
			$(".ui-tabs").tabs({
				fx: { opacity: "toggle", duration: "fast" }
			});
			
			$("input[type=text], textarea").each(function() {
				if ($(this).val() == $(this).attr("placeholder") || $(this).val() == "")
					$(this).css("color", "#999");
			});
			
			$("input[type=text], textarea").focus(function() {
				if ($(this).val() == $(this).attr("placeholder") || $(this).val() == "") {
					$(this).val("");
					$(this).css("color", "#000");
				}
			}).blur(function() {
				if ($(this).val() == "" || $(this).val() == $(this).attr("placeholder")) {
					$(this).val($(this).attr("placeholder"));
					$(this).css("color", "#999");
				}
			});
			
			$(".wrap h3, .wrap table").show();
		
			$(".warning").change(function() {
				if ($(this).is(":checked"))
					$(this).parent().css("background", "#c00").css("color", "#fff").css("fontWeight", "bold");
				else
					$(this).parent().css("background", "none").css("color", "inherit").css("fontWeight", "normal");
			});
			
			// Browser compatibility
			if ($.browser.mozilla) 
			         $("form").attr("autocomplete", "off");
		});
	</script>
</div>';
		
	}
	
	/**
	 * Description for section
	 *
	 * @since 1.0
	 */
	public function display_section() {
		// code
	}
	
	/**
	 * Description for About section
	 *
	 * @since 1.0
	 */
	public function display_about_section() {
		
		// This displays on the "About" tab. Echo regular HTML
		
	}
	
	/**
	 * HTML output for text field
	 *
	 * @since 1.0
	 */
	public function display_setting( $args = array() ) {
		
		extract( $args );
		
		$options = get_option( 'slashadmin_options' );
		
		if ( ! isset( $options[$id] ) && $type != 'checkbox' )
			$options[$id] = $std;
		elseif ( ! isset( $options[$id] ) )
			$options[$id] = 0;
		
		$field_class = '';
		$slash_ffe_check = ( function_exists('format_for_editor') ) ? format_for_editor( $options[$id] ) : wp_htmledit_pre( $options[$id] );
		if ( $class != '' )
			$field_class = ' ' . $class;
		
		switch ( $type ) {
			
			case 'heading':
				echo '</td></tr><tr valign="top"><td colspan="2"><h4>' . $desc . '</h4><p>' . $paragraph .'</p>';
				break;
			
			case 'checkbox':
				
				echo '<input class="checkbox' . $field_class . '" type="checkbox" id="' . $id . '" name="slashadmin_options[' . $id . ']" value="1" ' . checked( $options[$id], 1, false ) . ' /> <label for="' . $id . '">' . $desc . '</label>';
				
				break;
			
			case 'select':
				echo '<select class="select' . $field_class . '" name="slashadmin_options[' . $id . ']">';
				
				foreach ( $choices as $value => $label )
					echo '<option value="' . esc_attr( $value ) . '"' . selected( $options[$id], $value, false ) . '>' . $label . '</option>';
				
				echo '</select>';
				
				if ( $desc != '' )
					echo '<br /><span class="description">' . $desc . '</span>';
				
				break;

			case 'upload': // Media uploader (see: http://www.justinwhall.com/multiple-upload-inputs-in-a-wordpress-theme-options-page/)
				default:
					echo '<input id="' . $id . '" class="upload-url' . $field_class . '" type="text" name="slashadmin_options[' . $id . ']" value="' . esc_attr( $options[$id] ) . '" /><input id="st_upload_button" class="st_upload_button" type="button" name="upload_button" value="' . __( 'Upload', 'slashadmin' ) . '" />';
				
				if ( $desc != '' )
					echo '<span class="description">' . $desc . '</span>';
				
				break;
			
			case 'radio':
				$i = 0;
				foreach ( $choices as $value => $label ) {
					echo '<input class="radio' . $field_class . '" type="radio" name="slashadmin_options[' . $id . ']" id="' . $id . $i . '" value="' . esc_attr( $value ) . '" ' . checked( $options[$id], $value, false ) . '> <label for="' . $id . $i . '">' . $label . '</label>';
					if ( $i < count( $options ) - 1 )
						echo '<br />';
					$i++;
				}
				
				if ( $desc != '' )
					echo '<br /><span class="description">' . $desc . '</span>';
				
				break;
			
			case 'textarea':
				echo '<textarea class="' . $field_class . '" id="' . $id . '" name="slashadmin_options[' . $id . ']" placeholder="' . $std . '" rows="5" cols="30">' . $slash_ffe_check . '</textarea>';
				
				if ( $desc != '' )
					echo '<br /><span class="description">' . $desc . '</span>';
				
				break;
			
			case 'password':
				echo '<input class="regular-text' . $field_class . '" type="password" id="' . $id . '" name="slashadmin_options[' . $id . ']" value="' . esc_attr( $options[$id] ) . '" />';
				
				if ( $desc != '' )
					echo '<br /><span class="description">' . $desc . '</span>';
				
				break;
			
			case 'text':
			default:
		 		echo '<input class="regular-text' . $field_class . '" type="text" id="' . $id . '" name="slashadmin_options[' . $id . ']" placeholder="' . $std . '" value="' . esc_attr( $options[$id] ) . '" />';
		 		
		 		if ( $desc != '' )
		 			echo '<br /><span class="description">' . $desc . '</span>';
		 		
		 		break;
		 	
		}
		
	}


	/**
	 * Settings and defaults
	 * 
	 * @since 1.0
	 */
	public function get_settings() {
		
		/* Appearance
		===========================================*/

		$this->settings['opensans_fix'] = array( // Font fixes
			'section' => 'appearance',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Font fixes' , 'slashadmin' ),
			'type'    => 'heading'
		);
			$this->settings['default_opensans_remove'] = array(
				'section' => 'appearance',
				'title'   => __( 'Default Open Sans font' , 'slashadmin' ),
				'desc'    => __( 'Select how you should deal with the default Open Sans fonts that WordPress uses. If you have issues with how the font is displayed on logged-in users (it usually happens with non-latin subsets), try to disable it and then re-enter them properly below.' , 'slashadmin' ),
				'type'    => 'select',
				'std'     => 'default',
				'choices' => array(
					'default' 			=> __( 'Keep the default Wordpress fonts (default)' , 'slashadmin' ),
					'remove_frontend' 	=> __( 'Remove default Open Sans from the Frontend' , 'slashadmin' ),
					'remove_backend' 	=> __( 'Remove default Open Sans from both Frontend and Backend' , 'slashadmin' )
				)
			);
			$this->settings['custom_fonts'] = array(
				'section' => 'appearance',
				'title'   => __( 'Insert custom fonts' , 'slashadmin' ),
				'desc'    => __( 'Insert your Google Web Fonts here. You can declare more than one fonts, styles and language subsets. Keep in mind that you must only fill in the part after the "http://fonts.googleapis.com/css?family=" like for example <code>Open+Sans:400,300&subset=latin,greek</code>. View more information about the font\'s structure, subsets and weights at the <a href="https://developers.google.com/fonts/docs/getting_started#Overview" target="_blank">Google Fonts help page</a>. If your theme has it\'s own mechanism for embedding Google Fonts or if you prefer to do it manually, just leave it blank.' , 'slashadmin' ),
				'type'    => 'text',
				'std'     => '' 
			);

		$this->settings['cookielaw_header'] = array( // Cookie law
			'section' => 'appearance',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Cookie Law consent message' , 'slashadmin' ),
			'type'    => 'heading',
			'para' 	=> __( 'Show how your website complies with the EU Cookie Law. It displays a message at the bottom or the top of the page asking for the user\'s consent. Read more about <a href="http://ec.europa.eu/ipg/basics/legal/cookies/index_en.htm" target="_blank">EU legislation on cookies</a>', 'slashadmin' )
		);
			$this->settings['cookielaw_enable'] = array( // Hide update notices for all but Admins
				'section' => 'appearance',
				'title'   => __( 'Enable' , 'slashadmin' ),
				'desc'    => __( 'Show the Cookie Law consent message' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['cookielaw_style'] = array(
				'section' => 'appearance',
				'title'   => __( 'Message style' , 'slashadmin' ),
				'desc'    => __( 'Select the style of the message box.' , 'slashadmin' ),
				'type'    => 'select',
				'std'     => 'dark',
				'choices' => array(
					'dark' 		=> __( 'Dark (default)' , 'slashadmin' ),
					'light' 	=> __( 'Light' , 'slashadmin' ),
					'custom' 	=> __( 'None (I will style it my self)' , 'slashadmin' )
				)
			);
			$this->settings['cookielaw_position'] = array(
				'section' => 'appearance',
				'title'   => __( 'Position' , 'slashadmin' ),
				'desc'    => __( 'Select the position of the message box.' , 'slashadmin' ),
				'type'    => 'select',
				'std'     => 'bottom',
				'choices' => array(
					'bottom' 	=> __( 'Bottom' , 'slashadmin' ),
					'top'		=> __( 'Top' , 'slashadmin' ),
					'right' 	=> __( 'Bottom right' , 'slashadmin' )
				)
			);
			$this->settings['cookielaw_message'] = array(
				'section' => 'appearance',
				'title'   => __( 'Message' , 'slashadmin' ),
				'desc'    => __( 'Change the default message' , 'slashadmin' ),
				'type'    => 'text',
				'std'     => __( 'We use cookies. By browsing our site you agree to our use of cookies.', 'slashadmin' ) 
			);
			$this->settings['cookielaw_accept'] = array(
				'section' => 'appearance',
				'title'   => __( 'Accept text' , 'slashadmin' ),
				'desc'    => __( 'Change the "Accept" button text.' , 'slashadmin' ),
				'type'    => 'text',
				'std'     => __( 'Accept', 'slashadmin' )
			);
			$this->settings['cookielaw_readmore'] = array(
				'section' => 'appearance',
				'title'   => __( 'Read more' , 'slashadmin' ),
				'desc'    => __( 'Change the "read more" text' , 'slashadmin' ),
				'type'    => 'text',
				'std'     => __( 'Read more', 'slashadmin' )
			);

			$this->settings['cookielaw_url'] = array(
				'section' => 'appearance',
				'title'   => __( 'Read more page' , 'slashadmin' ),
				'desc'    => __( 'The page with the details about how your website uses cookies. If you don\'t select one, then no "read more" link will appear.' , 'slashadmin' ),
				'type'    => 'select',
				'std'     => 'none',
				'choices' => slashadmin_get_pages()
			);

		$this->settings['loading_header'] = array( // Loading animation
			'section' => 'appearance',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Loading animation' , 'slashadmin' ),
			'type'    => 'heading',
			'para' 	=> __( 'Show a "Loading" animation which dissapears when the page is fully loaded.', 'slashadmin' )
		);
			$this->settings['loading_enabled'] = array(
				'section' => 'appearance',
				'title'   => __( 'Enable loader' , 'slashadmin' ),
				'desc'    => __( 'Select whether the loader should be enabled or not. "Loader with a logo" displays a circular spinner with your logo in the middle. If no image is uploaded, it gets the site icon uploaded in the theme customizer. If no image is uploaded there either, it shows the spinner with no image (practically, in that case it is the same as the "Empty loader" option). "Custom GIF" option disables the circular spinner and allows you to upload and use your own animated GIF.' , 'slashadmin' ),
				'type'    => 'select',
				'std'     => 'disabled',
				'choices' => array(
					'disabled' 	=> __( 'Disabled' , 'slashadmin' ),
					'logo' 		=> __( 'Loader with a logo' , 'slashadmin' ),
					'gif' 		=> __( 'Custom GIF', 'slashadmin' ),
					'empty' 	=> __( 'Empty loader' , 'slashadmin' )
				)
			);
			$this->settings['loading_img'] = array(
				'title'   => __( 'Image loader', 'slashadmin' ),
				'desc'    => __( 'If you use a site icon (via <strong>Appearance &rarr; Customize &rarr; Site Identity &rarr; Site Icon</strong>) it will be used as the loader\'s image. In that case, just leave this field empty. If you want to display a different image as your loader, upload it here. The image should be square and no more than 160x160 pixels.', 'slashadmin' ) . '</span>',
				'std'     => '',
				'type'    => 'upload',
				'section' => 'appearance'
			);

			$this->settings['loading_bg_color'] = array(
				'section' => 'appearance',
				'title'   => __( 'Background color' , 'slashadmin' ),
				'desc'    => __( 'The loader\'s background color. You can use hex (e.g. <code>#FFFFFF</code>) or rgb/rgba (e.g. <code>rgba(255,255,255,0.8)</code>).' , 'slashadmin' ),
				'type'    => 'text',
				'std'     => '#FFFFFF' 
			);
			$this->settings['loading_spinner_empty'] = array(
				'section' => 'appearance',
				'title'   => __( 'Spinner background' , 'slashadmin' ),
				'desc'    => __( 'The background of the spinner.' , 'slashadmin' ),
				'type'    => 'text',
				'std'     => '#CCCCCC' 
			);
			$this->settings['loading_spinner_full'] = array(
				'section' => 'appearance',
				'title'   => __( 'Spinner loading color' , 'slashadmin' ),
				'desc'    => __( 'The color of the spinner\'s loading part.' , 'slashadmin' ),
				'type'    => 'text',
				'std'     => '#000000' 
			);
			$this->settings['loading_text'] = array(
				'section' => 'appearance',
				'title'   => __( 'Alt text' , 'slashadmin' ),
				'desc'    => __( 'The text to be displayed as the image\'s alt text.' , 'slashadmin' ),
				'type'    => 'text',
				'std'     => __( 'Loading...', 'slashadmin' ) 
			);
			$this->settings['loading_location'] = array(
				'section' => 'appearance',
				'title'   => __( 'Homepage only' , 'slashadmin' ),
				'desc'    => __( 'If checked, it will show the loader only on the homepage. Leave it unchecked to show it on all your website\'s pages. This option gets ignored if you insert the function manually.' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);

			$this->settings['loading_manual'] = array( 
				'section' => 'appearance',
				'title'   => __( 'Manual add' , 'slashadmin' ),
				'desc'    => __( 'Normally, the function is hooked to the footer via the <code>wp_footer</code> hook. For better results, you can disable that hook and manually add the loader to your template using <code>&lt;?php do_action(\'slash_admin_loader\'); ?&gt;</code>, prefferably right after your <code>&lt;body&gt;</code> opening.' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);

		$this->settings['old_ie_heading'] = array( // Compatibility
			'section' => 'appearance',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Compatibility' , 'slashadmin' ),
			'type'    => 'heading'
		);	
			$this->settings['old_browsers'] = array(
				'section' => 'appearance',
				'title'   => '', // Not used for headings.
				'desc'    => __( 'Old browser warning' , 'slashadmin' ),
				'type'    => 'heading'
			);
				$this->settings['oldie_warning'] = array( // Hide update notices for all but Admins
					'section' => 'appearance',
					'title'   => __( 'Old IE Warning' , 'slashadmin' ),
					'desc'    => __( 'If the visitor uses an old version of Internet Explorer (aka IE8 or older) display a message at the top of the page warning him that he is using an outdated browser and that he should update to a newer one.' , 'slashadmin' ),
					'type'    => 'checkbox',
					'std'     => 0 
				);
			
			$this->settings['legacy_options'] = array(
				'section' => 'appearance',
				'title'   => '', // Not used for headings.
				'desc'    => __( 'Legacy options' , 'slashadmin' ),
				'type'    => 'heading'
			);				
				$this->settings['favicon'] = array( // Favicon
					'title'   => __( 'Upload your favicon', 'slashadmin' ),
					'desc'    => '<span class="alert">' . __( 'Since WordPress 4.3, you can use a native Customizer option to upload your favicon (Under Appearance &rarr; Customize &rarr; Site Identity &rarr; Site Icon). This option here is kept for legacy reasons, but you should prefer the WordPress\' default.', 'slashadmin' ) . '</span>',
					'std'     => '',
					'type'    => 'upload',
					'section' => 'appearance'
				);

		/* Administration
		===========================================*/

		$this->settings['google_analytics'] = array(
			'section' => 'administration',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Google Analytics' , 'slashadmin' ),
			'type'    => 'heading'
		);
			$this->settings['analytics'] = array(
				'title'   => __( 'Analytics tracking code' , 'slashadmin' ),
				'desc'    => __( 'Add code for Google Analytics (it will be inserted before the <code>&lt;/head&gt;</code> tag).' , 'slashadmin' ),
				'std'     => '',
				'type'    => 'textarea',
				'section' => 'administration'
			);

		$this->settings['limit_revisions_header'] = array(
			'section' => 'administration',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Limit revisions' , 'slashadmin' ),
			'type'    => 'heading'
		);
			$this->settings['limit_revisions'] = array( // Limit Revisions
				'section' => 'administration',
				'title'   => __( 'Limit post revisions' , 'slashadmin' ),
				'desc'    => __( 'Limit the number of revisions that WordPress keeps for each post type. By default, an infinite number of revisions are stored if a post type supports revisions. Keep in mind that if you restrict this number, WordPress will purge the older revisions only after the post is updated.' , 'slashadmin' ),
				'type'    => 'select',
				'std'     => 'default',
				'choices' => array(
					'default' 	=> __( 'WordPress default (unlimited revisions)' , 'slashadmin' ),
					'one'		=> __( '1 revision (practically disables it)' , 'slashadmin' ),
					'two' 		=> __( '2 revisions' , 'slashadmin' ),
					'three'		=> __( '3 revisions' , 'slashadmin' ),
					'four'		=> __( '4 revisions' , 'slashadmin' ),
					'five'		=> __( '5 revisions' , 'slashadmin' ),
					'ten'		=> __( '10 revisions' , 'slashadmin' ),
					'twenty'	=> __( '20 revisions' , 'slashadmin' ),
					'fifty'		=> __( '50 revisions' , 'slashadmin' )
				)
			);

		$this->settings['prevent_editing'] = array( // Prevent updates on old posts
			'section' => 'administration',
			'title'   => __( 'Prevent updates on old posts' , 'slashadmin' ),
			'desc'    => __( 'Block post updates and deletion if the post is older than a specific period of time. Applies only to editors (admins can still edit the post as usual).' , 'slashadmin' ),
			'type'    => 'select',
			'std'     => 'default',
			'choices' => array(
				'default' 	=> __( 'Disabled' , 'slashadmin' ),
				'oneday' 	=> __( 'After one day' , 'slashadmin' ),
				'oneweek' 	=> __( 'After one week (7 days)' , 'slashadmin' ),
				'twoweeks'	=> __( 'After two weeks (14 days)' , 'slashadmin' ),
				'onemonth'	=> __( 'After one month (30 days)' , 'slashadmin' )
			)
		);

		$this->settings['maintenance_mode_header'] = array(
			'section' => 'administration',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Maintenance mode' , 'slashadmin' ),
			'type'    => 'heading'
		);
			$this->settings['maintenance_mode'] = array(
				'section' => 'administration',
				'title'   => __( 'Enable maintenance mode' , 'slashadmin' ),
				'desc'    => __( 'If checked, non-Admins will not be able to acess the WordPress backend and they will see the following message instead. Admins can always login as usual. Dont\' forget to un-check it back when you are done.' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 // Set to 1 to be checked by default, 0 to be unchecked by default.
			);
			$this->settings['maintenance_mode_msg'] = array(
				'title'   => __( 'Maintenance message' , 'slashadmin' ),
				'desc'    => __( 'The message to be displayed if an non-Admin tries to login while in maintenance mode.' , 'slashadmin' ),
				'std'     => sprintf( __( 'Site undergoing maintainance. Content uploading and editing will not be available for a while. If you have a question, please contact the website\'s <a href=\'mailto:%s\'>Administrator</a>.' , 'slashadmin' ), get_option('admin_email') ),
				'type'    => 'textarea',
				'section' => 'administration'
			);

		/* Login screen
		===========================================*/

		$this->settings['login_screen'] = array(
			'section' => 'frontend_usability',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Login screen' , 'slashadmin' ),
			'type'    => 'heading'
		);

		$this->settings['logo'] = array( // Logo
			'title'   => __( 'Upload your logo', 'slashadmin' ),
			'desc'    => __( 'Upload the logo to appear at the login screen of your website. The plugin will try to automatically adjust the width/height but the suggested dimensions are <strong>320x80 pixels</strong>.', 'slashadmin' ),
			'std'     => '',
			'type'    => 'upload',
			'section' => 'frontend_usability'
		);

		$this->settings['login_links'] = array(
			'section' => 'frontend_usability',
			'title'   => __( 'Fix links at the login screen' , 'slashadmin' ),
			'desc'    => __( 'Fix the links at the wordpress login screen in order to link to your website instead of wordpress.org.' , 'slashadmin' ),
			'type'    => 'checkbox',
			'std'     => 1 // Set to 1 to be checked by default, 0 to be unchecked by default.
		);

		$this->settings['homepage_redirect'] = array( // Redirect to the homepage after login
			'section' => 'frontend_usability',
			'title'   => __( 'Go to homepage after login' , 'slashadmin' ),
			'desc'    => __( 'After log-in, redirect users to the website\'s homepage instead of their profile page.' , 'slashadmin' ),
			'type'    => 'checkbox',
			'std'     => 0 
			);

		$this->settings['disable_adminbar'] = array( // Disable Admin Bar for non Admins.
			'section' => 'frontend_usability',
			'title'   => __( 'Disable Admin Bar for non Admins' , 'slashadmin' ),
			'desc'    => __( 'Disable the Admin Bar for all users except Administrators (applies only to the front-end).' , 'slashadmin' ),
			'type'    => 'checkbox',
			'std'     => 0 
			);
		$this->settings['login_css'] = array(
			'section' => 'frontend_usability',
			'title'   => __( 'Custom CSS' , 'slashadmin' ),
			'desc'    => __( 'Your custom styles for the login screen (quick referrence for tags you might want to use: <code>body.login.login-action-login, div#login, div#login h1 a, form#loginform, input#user_login.input, input#user_pass.input, input#wp-submit, p#nav, p#backtoblog</code>).' , 'slashadmin' ),
			'type'    => 'textarea'
		);

		/* Backend usability
		===========================================*/

		$this->settings['restrict_access'] = array(
			'section' => 'backend_usability',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Hide options for non-admins' , 'slashadmin' ),
			'type'    => 'heading'
		);

			$this->settings['exclude_media'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Media' , 'slashadmin' ),
				'desc'    => __( 'Exclude media' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['exclude_comments'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Comments' , 'slashadmin' ),
				'desc'    => __( 'Exclude comments' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['exclude_tools'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Tools' , 'slashadmin' ),
				'desc'    => __( 'Exclude tools' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['exclude_profile'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Profile' , 'slashadmin' ),
				'desc'    => __( 'Exclude profile' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['exclude_posts'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Posts' , 'slashadmin' ),
				'desc'    => __( 'Exclude posts' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['exclude_pages'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Pages' , 'slashadmin' ),
				'desc'    => __( 'Exclude pages' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['exclude_jetpack'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Jetpack' , 'slashadmin' ),
				'desc'    => __( 'Exclude Jetpack plugin' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['exclude_custom'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Custom excludes' , 'slashadmin' ),
				'desc'    => __( 'Exclude other options such as custom post types or even the Dashboard. To do so, add the names separated with commas and no blank spaces like for example <code>Dashboard,Pages,Media,Posts</code> etc. You can exclude <strong>up to 5 additional items</strong>.' , 'slashadmin' ),
				'type'    => 'text',
				'std'     => '' 
			);

			$this->settings['css_excludes'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Exclude by CSS' , 'slashadmin' ),
				'desc'    => __( 'If there is an section that you cannot hide with the options above, you can hide it by adding here its CSS id or class. You can hide multiple elements by separating their ids or classes with comma (e.g.: <code>#menu-item-id, #menu-item-2-id, .menu-item-3-class, .menu-item-4-class</code>). Please keep in mind that this only hides the options with CSS. You should only use it for usability purposes and not for security reasons.' , 'slashadmin' ),
				'type'    => 'text'
			);

			$this->settings['hide_update_notices'] = array( // Hide update notices for all but Admins
				'section' => 'backend_usability',
				'title'   => __( 'Hide update notices to non Admins' , 'slashadmin' ),
				'desc'    => __( 'Hide the notices for updating Wordpress and other plugins for all users except from Admins' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);

		$this->settings['editors_allow_header'] = array( // Show extra options to Editors
			'section' => 'backend_usability',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Show extra options to Editors' , 'slashadmin' ),
			'type'    => 'heading'
		);
			$this->settings['editors_allow_menus'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Menus' , 'slashadmin' ),
				'desc'    => __( 'Allow Editors to manage the website\'s menus.' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['editors_allow_widgets'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Widgets' , 'slashadmin' ),
				'desc'    => __( 'Allow Editors to manage the website\'s widgets.' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['editors_allow_customizer'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Customizer' , 'slashadmin' ),
				'desc'    => __( 'Allow Editors access to the Theme Customizer.' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['editors_allow_background'] = array(
				'section' => 'backend_usability',
				'title'   => __( 'Background' , 'slashadmin' ),
				'desc'    => __( 'Allow Editors to change the website\'s background (check this only if your theme has support for WordPress\' <a href="http://codex.wordpress.org/Function_Reference/add_theme_support#Custom_Background" target="_blank">Custom Background</a> feature).' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);

		/* White label
		===========================================*/
		$this->settings['white_label_header'] = array(
			'section' => 'white_label',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'White label Backend' , 'slashadmin' ),
			'type'    => 'heading'
		);
			$this->settings['howdy'] = array(
				'section' => 'white_label',
				'title'   => __( 'Change "Howdy"' , 'slashadmin' ),
				'desc'    => __( 'Change the "Howdy" message at the top right corner of the screen.' , 'slashadmin' ),
				'type'    => 'text',
				'std'     => '' 
			);
			$this->settings['footer_txt'] = array(
				'section' => 'white_label',
				'title'   => __( 'Change footer text' , 'slashadmin' ),
				'desc'    => __( 'Change the footer text (you can use HTML markup).' , 'slashadmin' ),
				'type'    => 'textarea'
			);
			$this->settings['admin_logo'] = array( // Logo
				'title'   => __( 'Change the Admin logo', 'slashadmin' ),
				'desc'    => __( 'Replace the WordPress logo at the top left of the screen. Suggested image dimension is <strong>92x92 pixels</strong>.', 'slashadmin' ),
				'std'     => '',
				'type'    => 'upload',
				'section' => 'white_label'
			);

			$this->settings['dashboard_welcome_header'] = array(
				'section' 	=> 'white_label',
				'title'   	=> '', // Not used for headings.
				'desc'    	=> __( 'Welcome message' , 'slashadmin' ),
				'type'    	=> 'heading',
				'para' 		=> __( 'Change the Dashboard welcome message. Removes the default welcome message at the Dashboard and replaces it with your own.', 'slashadmin' )
			);
				$this->settings['dashboard_welcome'] = array(
					'section' => 'white_label',
					'title'   => __( 'Your welcome message' , 'slashadmin' ),
					'desc'    => __( 'Add the content of your custom welcome message (HTML markup is allowed).' , 'slashadmin' ),
					'type'    => 'textarea'
				);

			$this->settings['dashboard_widget_header'] = array(
				'section' 	=> 'white_label',
				'title'   	=> '', // Not used for headings.
				'desc'    	=> __( 'Add a Dashboard widget' , 'slashadmin' ),
				'type'    	=> 'heading',
				'para' 		=> __( 'Add a Dashboard Widget to provide general or commercial information to your clients (for example: your contact info or links to support documentation).', 'slashadmin' )
			);
				$this->settings['dashboard_widget_title'] = array(
					'section' => 'white_label',
					'title'   => __( 'Widget title' , 'slashadmin' ),
					'desc'    => __( 'Add the title of the Dashboard widget.' , 'slashadmin' ),
					'type'    => 'text',
					'std'     => '' 
				);
				$this->settings['dashboard_widget_content'] = array(
					'section' => 'white_label',
					'title'   => __( 'Widget content' , 'slashadmin' ),
					'desc'    => __( 'Add the content of the Dashboard widget (HTML markup is allowed).' , 'slashadmin' ),
					'type'    => 'textarea'
				);

			$this->settings['admin_css_header'] = array(
				'section' 	=> 'white_label',
				'title'   	=> '', // Not used for headings.
				'desc'    	=> __( 'Custom CSS' , 'slashadmin' ),
				'type'    	=> 'heading'
			);
				$this->settings['admin_css'] = array(
					'section' => 'white_label',
					'title'   => __( 'Custom CSS for the Admin' , 'slashadmin' ),
					'desc'    => __( 'The styles that you put here affect only the Admin and they don\'t get called at the frontend. Quick referrence for tags you might want to use: <code>div#slashadmin_dashboard_widget</code> for the custom widget, <code>div#welcome-panel</code> for the welcome panel.' , 'slashadmin' ),
					'type'    => 'textarea'
				);

		/* Performance
		===========================================*/
		$this->settings['dns_prefetching_header'] = array(
			'section' => 'performance',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'DNS prefetching' , 'slashadmin' ),
			'type'    => 'heading'
		);
			$this->settings['dns_prefetch'] = array(
				'section' => 'performance',
				'title'   => __( 'DNS prefetching' , 'slashadmin' ),
				'desc'    => __( 'DNS prefetching notifies the client that there are assets we\'ll need later from a specific URL (outside our website\'s domain) so the browser can resolve the DNS as quickly as possible (<a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Controlling_DNS_prefetching" target="_blank">read more</a>). Enter here the  URLs to be prefetched (each URL in separate line). Here\'s some examples of sources that can be prefetched for an easy copy/paste: 
					</br><code>//fonts.googleapis.com</code> (Google web fonts)</br>
					<code>//google-analytics.com</code> & <code>//www.google-analytics.com</code> (Google Analytics)</br>
					<code>//platform.twitter.com</code> (Twitter)</br>
					<code>//maxcdn.bootstrapcdn.com</code> (MaxCDN, used by Font Awesome)' , 'slashadmin' ),
				'type'    => 'textarea'
			);

		$this->settings['prefetch_prerender_header'] = array(
			'section' => 'performance',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Prefetch and prerender pages' , 'slashadmin' ),
			'type'    => 'heading',
			'para'	=> __( 'Link prefetching is a browser mechanism, which utilizes browser idle time to download or prefetch documents that the user might visit in the near future. A web page provides a set of prefetching hints to the browser, and after the browser is finished loading the page, it begins silently prefetching specified documents and stores them in its cache. When the user visits one of the prefetched documents, it can be served up quickly out of the browser\'s cache. Prerendering downloads and renders the entire page and hides it from the user until it is requested, therefore, it should be used with caution.', 'slashadmin' ) 
		);
			$this->settings['prefetch_next'] = array(
				'section' => 'performance',
				'title'   => __( 'Prefetch next page' , 'slashadmin' ),
				'desc'    => __( 'Prefetch next page when on archives.' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['prerender_next'] = array(
				'section' => 'performance',
				'title'   => __( 'Prerender next page' , 'slashadmin' ),
				'desc'    => __( 'Prerender next page when on archives.' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['prefetch_home'] = array(
				'section' => 'performance',
				'title'   => __( 'Prefetch homepage' , 'slashadmin' ),
				'desc'    => __( 'Prefetch homepage when on sigle posts and pages.' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);
			$this->settings['prerender_home'] = array(
				'section' => 'performance',
				'title'   => __( 'Prerender homepage' , 'slashadmin' ),
				'desc'    => __( 'Prerender homepage when on sigle posts and pages.' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);

		/* Shortcodes
		===========================================*/
		$this->settings['old_browsers'] = array(
			'section' => 'shortcodes',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Shortcodes' , 'slashadmin' ),
			'type'    => 'heading'
		);
			$this->settings['shortcode_mail'] = array( // Hide update notices for all but Admins
				'section' => 'shortcodes',
				'title'   => __( 'Protect mail from harvesters' , 'slashadmin' ),
				'desc'    => __( 'To disguise an email address, use <code>[slash_mail address="yourmail@mail.com"]</code>.<br />Use <code>[slash_mailto address="yourmail@mail.com"]</code> instead if you want to automatically make it a link as well. Using just <code>[slash_mail]</code> or <code>[slash_mailto]</code> will automatically display the post/page author\'s email address. For a more detailed explanation click on the Documentation tab.' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);

			$this->settings['shortcode_url'] = array( // Hide update notices for all but Admins
				'section' => 'shortcodes',
				'title'   => __( 'Enable relative URLs' , 'slashadmin' ),
				'desc'    => __( 'If you develop your site on localhost or on a temporary URL, you might want to avoid absolute URLs inside posts and pages. That way you don\'t need to update your links after migrating to your actual domain.  The available options are:<br />
					<code>[slash_home]</code> retrieves the <a href="http://codex.wordpress.org/Function_Reference/home_url" target="_blank">home URI</a> for the current site.<br />
					<code>[slash_theme]</code> retrieves the <a href="http://codex.wordpress.org/Function_Reference/get_template_directory_uri" target="_blank">template directory URI</a> for the current theme.<br />
					<code>[slash_child]</code> retrieves the <a href="http://codex.wordpress.org/Function_Reference/get_stylesheet_directory_uri" target="_blank">stylesheet directory URI</a> for the current theme/child theme.<br />
					Then, you can use them like so <code>&lt;img src="[slash_child]/images/image.png" /&gt;</code>, with which you would get <code>&lt;img src="http://www.yourdomain.com/wp-content/themes/childtheme/images/image.png" /&gt;</code>' , 'slashadmin' ),
				'type'    => 'checkbox',
				'std'     => 0 
			);		

		/* FAQ
		===========================================*/
		
		$this->settings['q0_heading'] = array(
			'section' => 'about',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'My theme also supports some of this plugin\'s features. Which one should I choose?' , 'slashadmin' ),
			'type'    => 'heading',
			'para'	  => __( 'It is up to you to decide whether you will use your theme\'s options or those provided by this plugin. It is recommended, though, that you keep those settings separated from your theme and the reason is simple: If at some point you decide to switch themes, those options will be lost and you have to remember to re-enter them. Keeping them in a plugin maintains the options between themes.' , 'slashadmin' )
		);

		$this->settings['q1_1_heading'] = array(
			'section' => 'about',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Which shortcodes are available?' , 'slashadmin' ),
			'type'    => 'heading',
			'para'	  => __( 'You can enable the following shortcodes:
								<br />- <code>[slash_mail address="yourmail@mail.com"]</code>. If you manually include email addresses in your posts, you should consider disguising them in order to "fool" e-mail harvesters. This shortcode displays the "disguised" version of the given email as plain text.
								<br />- <code>[slash_mailto address="yourmail@mail.com"]</code> does the exact same thing, but it also transforms the text to a "mailto" link as well. 
								<br />- <code>[slash_mail]</code> and <code>[slash_mailto]</code> will automatically display the post/page author\'s email address. All the above shortcodes take advantage of the <code>&lt;?php echo antispambot(); ?&gt;</code> function. For a more in-depth explanation on why you should care about disquising your emails, check the <a href="http://codex.wordpress.org/Protection_From_Harvesters" target="_blank">WordPress Codex</a>.
								<br />- <code>[slash_home]</code> retrieves the <a href="http://codex.wordpress.org/Function_Reference/home_url" target="_blank">home URI</a> for the current site. It is the equivalent of <code>&lt;?php echo home_url(); ?&gt;</code>. 
								<br />- <code>[slash_theme]</code> retrieves the <a href="http://codex.wordpress.org/Function_Reference/get_template_directory_uri" target="_blank">template directory URI</a> for the current theme. It is the equivalent of <code>&lt;?php echo get_template_directory_uri(); ?&gt;</code>.
								<br />- <code>[slash_child]</code> retrieves the <a href="http://codex.wordpress.org/Function_Reference/get_stylesheet_directory_uri" target="_blank">stylesheet directory URI</a> for the current theme/child theme. It is the equivalent of <code>&lt;?php echo get_stylesheet_directory_uri(); ?&gt;</code>.
						' , 'slashadmin' )
		);

		$this->settings['q1_heading'] = array(
			'section' => 'about',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'What\'s wrong with the default WordPress Open Sans fonts?' , 'slashadmin' ),
			'type'    => 'heading',
			'para'	  => __( 'Since version 3.8 WordPress uses Open Sans Web Font in the admin section and on all public pages when a user is logged in (for the admin bar). Some times this causes an issue when you want to use another configuration of Open Sans for your site. If you use Open Sans in your theme and you notice inconsistencies in the font\'s display between logged in and anonymous users or if you just don\'t want Open Sans for some reason (e.g. for better speed or privacy concerns), you can disable the WordPress\' default configuration. Then, you can insert your own configuration of Open Sans either via this plugin, your theme\'s options panel or manually.' , 'slashadmin' )
		);

		$this->settings['q2_heading'] = array(
			'section' => 'about',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'What does hiding options for non-admins means?' , 'slashadmin' ),
			'type'    => 'heading',
			'para'	  => __( 'Sometimes you only use certain features of Wordpress. For example, your website might have comments disabled or not using a certain feature. Also, for better usability you might want to show your users only the options that concern them. Hiding those options won\'t remove them. You, as an administrator, will always see the full list of all the available options. An editor, though, won\'t see the hidden options, which helps him focus to only those that concern him.' , 'slashadmin' )
		);

		$this->settings['q3_heading'] = array(
			'section' => 'about',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Is it wise to hide update notices from my users?' , 'slashadmin' ),
			'type'    => 'heading',
			'para'	  => __( 'Generally speaking, no. Wordpress\' default behaviour is probably the best, that\'s why the specific option is disabled by default. In some cases, though, users might get confused with those notifications or think that something is wrong with the website. In cases like that, you might want to keep the update notifications visible only for those who can apply them - namely the administrators. Keep in mind that, technically, selecting this option won\'t remove the notifications for the non-admins - it will just hide them via CSS.' , 'slashadmin' )
		);

		$this->settings['q4_heading'] = array(
			'section' => 'about',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'How does allowing access to appearance settings work?' , 'slashadmin' ),
			'type'    => 'heading',
			'para'	  => __( 'You can allow editors access to one or more of the following sub-sections of the "Appearance" section: <br />- Customize<br />- Widgets<br />- Menus<br />- Background<br />Technically, by selecting even one of the above options you give editors access to the Appearance section. To prevent them from accessing unwanted subsections (e.g. you want them to see the Menus but not the Widgets) the plugin hides their links via CSS/JavaScript from both the backend and the frontend. If an editor knew the link for the Widgets subsection he/she could access it. By default the plugin respects the WordPress\' default behavior, keeping those options disabled (users have no access at all to the Appearance section).'  , 'slashadmin' )
		);

		$this->settings['q5_heading'] = array(
			'section' => 'about',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Old browser warning behaves strangely with W3TC plugin' , 'slashadmin' ),
			'type'    => 'heading',
			'para'	  => __( 'This is a known issue. When Page Caching is activated in the W3 Total Cache plugin, the old browser warning becomes unpredictable and it may appear not only in Internet Explorer 8 but in newer IE versions as well as in Chrome. To deal with the problem you need to disable either the old IE warning or the W3TC Page Cache option.'  , 'slashadmin' )
		);

		$this->settings['q6_heading'] = array(
			'section' => 'about',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'Which are the minimum requirements to run this plugin?' , 'slashadmin' ),
			'type'    => 'heading',
			'para'	  => __( 'This plugin has been tested with Wordpress 4.0 and above. It might probably work with older versions too, but you should always use the latest version of Wordpress.'  , 'slashadmin' )
		);
		$this->settings['q7_heading'] = array(
			'section' => 'about',
			'title'   => '', // Not used for headings.
			'desc'    => __( 'I found a bug / I have a suggestion or a feature request.' , 'slashadmin' ),
			'type'    => 'heading',
			'para'	  => __( 'You can post a message at the plugin\'s <a target="_blank" href="http://wordpress.org/support/plugin/slash-admin">support forum</a> and I will to my best to help you out.'  , 'slashadmin' )
		);	
	}
	
	/**
	 * Initialize settings to their default values
	 * 
	 * @since 1.0
	 */
	public function initialize_settings() {
		
		$default_settings = array();
		foreach ( $this->settings as $id => $setting ) {
			if ( $setting['type'] != 'heading' )
				$default_settings[$id] = isset($setting['std']) ? $setting['std'] : '';
		}
		
		update_option( 'slashadmin_options', $default_settings );
		
	}
	
	/**
	* Register settings
	*
	* @since 1.0
	*/
	public function register_settings() {
		
		register_setting( 'slashadmin_options', 'slashadmin_options', array ( &$this, 'validate_settings' ) );
		
		foreach ( $this->sections as $slug => $title ) {
			if ( $slug == 'about' )
				add_settings_section( $slug, $title, array( &$this, 'display_about_section' ), 'slashadmin-options' );
			else
				add_settings_section( $slug, $title, array( &$this, 'display_section' ), 'slashadmin-options' );
		}
		
		$this->get_settings();
		
		foreach ( $this->settings as $id => $setting ) {
			$setting['id'] = $id;
			$this->create_setting( $setting );
		}
		
	}
	
	/**
	* jQuery Tabs
	*
	* @since 1.0
	*/
	public function slashadmin_scripts() {
		
	wp_print_scripts( 'jquery-ui-tabs' );
	wp_enqueue_media();
	wp_register_script('slashadmin-upload', plugins_url( 'js/uploader.js' , __FILE__ ) , array('jquery','media-upload','thickbox'));
		$params = array(
			'txt'	=> __( 'Use this image', 'slashadmin' )
		);
		wp_localize_script( 'slashadmin-upload', 'uploader', $params );

	wp_enqueue_script('slashadmin-upload');

	//Media Uploader Style
	wp_enqueue_style('thickbox');
		
	}
	
	/**
	* Styling for the plugin options page
	*
	* @since 1.0
	*/
	public function styles() {
		
		wp_register_style( 'slashadmin-admin', plugins_url( 'css/slashadmin-options.css' , __FILE__ ) );
		wp_enqueue_style( 'slashadmin-admin' );
		
	}
	
	/**
	* Validate settings
	*
	* @since 1.0
	*/
	public function validate_settings( $input ) {
		
		if ( ! isset( $input['reset_plugin'] ) ) {
			$options = get_option( 'slashadmin_options' );
			
			foreach ( $this->checkboxes as $id ) {
				if ( isset( $options[$id] ) && ! isset( $input[$id] ) )
					unset( $options[$id] );
			}
			
			return $input;
		}
		return false;
		
	}
	
}

$slashadmin_options = new Slash_Admin_Options();

?>